#include "ntp.h"
#include "sensor_console.h"
#include "web_config.h"
#include <WiFiUdp.h> // required for NTP
#include "src/lib/NTPClient/NTPClient.h" // NTP

//NOTE: ESP32 sometimes couldn't access the NTP server, and every loop would take +1000ms
// ifdefs could be used to define functions specific to ESP32, e.g. with configTime
namespace ntp {
  WiFiUDP ntpUDP;
  NTPClient timeClient(ntpUDP);
  bool connected_at_least_once = false;
  void setLocalTime(int32_t unix_seconds);

  // Should be defined, even offline
  void initialize() {
    timeClient.setTimeOffset((config::time_zone + config::daylight_saving_time) * 3600);
    sensor_console::defineIntCommand("set_time", ntp::setLocalTime, F("1618829570 (Sets time to the given UNIX time)"));
  }

  void connect(){
    timeClient.setPoolServerName(config::ntp_server);
    timeClient.setUpdateInterval(60000UL);
    Serial.print("NTP - Trying to connect to : ");
    Serial.println(config::ntp_server);
    timeClient.begin();
  }

  void update() {
    connected_at_least_once |= timeClient.update();
  }

  void getLocalTime(char *timestamp) {
    timeClient.getFormattedDate(timestamp);
  }

  void setLocalTime(int32_t unix_seconds) {
    char time[23];
    timeClient.getFormattedDate(time);
    Serial.print(F("Current time : "));
    Serial.println(time);
    if (connected_at_least_once) {
      Serial.println(F("NTP update already happened. Not changing anything."));
      return;
    }
    Serial.print(F("Setting UNIX time to : "));
    Serial.println(unix_seconds);
    timeClient.setEpochTime(unix_seconds - seconds());
    timeClient.getFormattedDate(time);
    Serial.print(F("Current time : "));
    Serial.println(time);
  }
}
